#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <utime.h>
#include <sys/stat.h>
#ifndef CRAY
#include <sys/time.h>
#else
#include <bsd/bsdutime.h>
#endif

/* If PROGRAM is defined, a timetransfer program is compiled with
 *	the timetransfer function.
 */

/*
 *      Copyright (c) 1993 by California Institute of Technology.
 *      Written by William Deich.  Not derived from licensed software.

 *      Permission is granted to anyone to use this software for any
 *      purpose on any computer system, and to redistribute it freely,
 *      subject to the following restrictions:
 *
 *      1. The author is not responsible for the consequences of use of this
 *         software, no matter how awful, even if they arise from defects in it.
 *
 *      2. The origin of this software must not be misrepresented, either
 *         by explicit claim or by omission.
 *
 *      3. Altered versions must be plainly marked as such, and must not
 *         be misrepresented as being the original software.
 */



int
timetransfer(fromfile, tofile, delta_s, delta_us)
char *fromfile;
char *tofile;
long delta_s, delta_us;
{
    /* Transfers the mtime from fromfile to tofile,
     * adding delta_s sec + delta_us usec to the mtime.
     * Returns -1 on error.
     * If fromfile == "=", it means to use "tofile".
     */

    extern errno;
    struct stat statfrom, statto;
    struct timeval am_times[2];
    struct utimbuf times;

    if (stat(tofile, &statto) == -1) {
	int e = errno;
	(void) fprintf(stderr, "timetransfer(): stat failed on %s: ", tofile);
	errno = e;
	perror("");
	return -1;
    }

    if (strcmp(fromfile, tofile) == 0 || strcmp(fromfile, "=") == 0) {
	statfrom = statto;
    } else {
	if (stat(fromfile, &statfrom) == -1) {
	    int e = errno;
	    (void) fprintf(stderr,
		"timetransfer(): stat failed on %s: ", fromfile);
	    errno = e;
	    perror("");
	    return -1;
	}
    }

    /* Don't change the access time: */
    am_times[0].tv_sec = statto.st_atime;
    am_times[0].tv_usec = 0L;

    /* Change the modification time to that of fromfile: */
    am_times[1].tv_sec = statfrom.st_mtime + delta_s;
    am_times[1].tv_usec = 0L + delta_us;
    if (am_times[1].tv_usec > 1000000) {
	am_times[1].tv_sec++;
	am_times[1].tv_usec -= 1000000;
    }

#ifdef UTIMES
    if (utimes(tofile, am_times) == -1 ) {
	int e = errno;
	(void) fprintf(stderr, "timetransfer(): Couldn't change times on %s: ",
		tofile);
	errno = e;
	perror("");
	return -1;
    }
#else
    times.actime = am_times[0].tv_sec;
    times.modtime = am_times[1].tv_sec +
		am_times[1].tv_usec/1.e6 + 0.5;	/* +0.5 rounds to nearest sec */
    if (utime(tofile, &times) == -1 ) {
	int e = errno;
	(void) fprintf(stderr, "timetransfer(): Couldn't change times on %s: ",
		tofile);
	errno = e;
	perror("");
	return -1;
    }
#endif
    return 0;
}

#ifdef PROGRAM
/*
 * $0 [ +/-delta-seconds ] fromfile tofile...
 * Transfers the mtime from fromfile to tofile, adding delta-seconds if present.
 * Delta-seconds is float number of seconds.  Fractional seconds are
 * supported to the limit imposed by the OS.

 * If fromfile == "=", it means to use "tofile" for each of the tofile's.
 */

char version[] = VERSION "." PATCHLEVEL ;

int errno;

char *prog;

main(argc, argv)
int argc;
char **argv;
{
    char **av;
    char *s, *fromfile;
    char *strrchr();

    int n;
    float f;
    long delta_s = 0;
    long delta_us = 0;
    int timetransfer();

    prog = argv[0];
    if ( (s=strrchr(prog, '/')) && s != prog+strlen(prog)-1 ) prog = s+1;

    av = &argv[1];
    if (*av && (**av == '-' || **av == '+')) {
	n = sscanf(*av++, "%f", &f);
	if (n == 0) {
	    (void) fprintf(stderr, "%s: Invalid delta-seconds `%s'\n",
		    prog, *--av);
	    (void) exit(1);
	}
	delta_s = (long) f;
	delta_us = (f - (float) delta_s) * 1000000;
    }
    if (*av)
	fromfile = *av++;

    if (*av == NULL) {
	(void) fprintf(stderr,
		"Usage: %s [+/-deltasec] fromfile tofile ...\n",
			prog);
	(void) exit(1);
    }

    while (*av)
	(void) timetransfer(fromfile, *av++, delta_s, delta_us);
    (void) exit(0);
}
#endif
